#import grok
import grokcore.component
import zope.schema
import sqlalchemy.types
from zope.schema.interfaces import IField
from zope.interface import Interface
from zope.interface.interface import InterfaceClass
from zope.interface.interfaces import IInterface

#def Fields(model):
#    return grok.Fields(schema_from_model(model))

def schema_from_model(model):
    table = model.__table__
    bases = (Interface,)
    attrs = {}
    for i, column in enumerate(table.columns):
        if len(column.foreign_keys) or column.primary_key:
            continue
        field = IField(column.type)
        field.__name__ = str(column.name)
        field.title = unicode(column.name)
        field.required = not column.nullable
        attrs[column.name] = field
        field.order = i
       
    return InterfaceClass(name=model.__table__.name,
                          bases=bases,
                          attrs=attrs,
                          __doc__='Generated from metadata')

@grokcore.component.adapter(sqlalchemy.types.String)
@grokcore.component.implementer(IField)
def field_from_sa_string(s):
    return zope.schema.TextLine(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Text)
@grokcore.component.implementer(IField)
def field_from_sa_text(s):
    return zope.schema.Text(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Integer)
@grokcore.component.implementer(IField)
def field_from_sa_integer(i):
    return zope.schema.Int(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Float)
@grokcore.component.implementer(IField)
def field_from_sa_float(i):
    return zope.schema.Float(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.DateTime)
@grokcore.component.implementer(IField)
def field_from_sa_datetime(i):
    return zope.schema.Datetime(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Date)
@grokcore.component.implementer(IField)
def field_from_sa_date(i):
    return zope.schema.Date(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Time)
@grokcore.component.implementer(IField)
def field_from_sa_time(i):
    return zope.schema.Time(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Interval)
@grokcore.component.implementer(IField)
def field_from_sa_interval(i):
    return zope.schema.Timedelta(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Boolean)
@grokcore.component.implementer(IField)
def field_from_sa_boolean(i):
    return zope.schema.Bool(__name__ = u'__dummy__',
                title = u'__dummy__')

@grokcore.component.adapter(sqlalchemy.types.Binary)
@grokcore.component.implementer(IField)
def field_from_sa_binary(i):
    return zope.schema.Bytes(__name__ = u'__dummy__',
                title = u'__dummy__')



"""
We test an initialization pattern for z3c.saconfig where the metadata
is set up in an event handler.

Let's first grok things::

  >>> grok.testing.grok('megrok.rdb.meta')
  >>> grok.testing.grok(__name__)

We start using the database now::

  >>> session = rdb.Session()

Let's start using the database now::

  >>> session = rdb.Session()
  >>> philosophy = Department(name='Philosophy')
  >>> session.add(philosophy)
  >>> logic = Course(name='Logic')
  >>> ethics = Course(name='Ethics')
  >>> metaphysics = Course(name='Metaphysics')
  >>> session.add_all([logic, ethics, metaphysics])
  
Let's now add them to the courses container::

  >>> philosophy.courses.set(logic)
  >>> philosophy.courses.set(ethics)
  >>> philosophy.courses.set(metaphysics)

We can now verify that the courses are there::

  >>> [(course.id, course.name, course.department_id) for course in
  ... session.query(Course)]
  [(1, 'Logic', 1), (2, 'Ethics', 1), (3, 'Metaphysics', 1)]

  >>> for key, value in sorted(philosophy.courses.items()):
  ...   print key, value.name, value.department.name
  1 Logic Philosophy
  2 Ethics Philosophy
  3 Metaphysics Philosophy
"""

import grokcore.component as grok
from z3c.saconfig import EngineFactory, GloballyScopedSession
from z3c.saconfig.interfaces import IEngineCreatedEvent

from sqlalchemy import Column, ForeignKey
from sqlalchemy.types import Integer, String
from sqlalchemy.orm import relation

from megrok import rdb

TEST_DSN = 'sqlite:///:memory:'

engine_factory = EngineFactory(TEST_DSN)
scoped_session = GloballyScopedSession()

grok.global_utility(engine_factory, direct=True)
grok.global_utility(scoped_session, direct=True)

metadata = rdb.MetaData()

@grok.subscribe(IEngineCreatedEvent)
def setUpDatabase(event):
    rdb.setupDatabase(metadata)

rdb.metadata(metadata)

class Courses(rdb.Container):
    pass

class Department(rdb.Model):
    id = Column('id', Integer, primary_key=True)
    name = Column('name', String(50))
    courses = relation('Course', 
                       backref='department',
                       collection_class=Courses)

class Course(rdb.Model):
    id = Column('id', Integer, primary_key=True)
    department_id = Column('department_id', Integer, 
                           ForeignKey('department.id'))
    name = Column('name', String(50))

